import sbt._
import Settings._

enablePlugins(DockerComposePlugin)

lazy val assmSettings =
  assemblyMergeStrategy in assembly := {
    case x if x.endsWith(".class") => MergeStrategy.last
    case x if x.endsWith(".properties") => MergeStrategy.last
    case x if x.contains("/resources/") => MergeStrategy.last
    case x if x.startsWith("META-INF/mailcap") => MergeStrategy.last
    case x if x.startsWith("META-INF/mimetypes.default") => MergeStrategy.first
    case x if x.startsWith("META-INF/maven/org.slf4j/slf4j-api/pom.") => MergeStrategy.first
    case x =>
      val oldStrategy = (assemblyMergeStrategy in assembly).value
      if (oldStrategy == MergeStrategy.deduplicate)
        MergeStrategy.first
      else
        oldStrategy(x)
  }


docker <<= (docker in backendapi,docker in frontendapi) map {(image, _) => image}

dockerImageCreationTask := docker.value

val dockerAppPath = "/app/"


lazy val root = project.root
  .setName("SBT template")
  .setDescription("Backup DSL")
  .setInitialCommand("_")
  .configureRoot
  .aggregate(common, frontendapi,backendapi)

lazy val common = project.from("common")
  .setName("common")
  .setDescription("Common utilities")
  .setInitialCommand("_")
  .configureModule
  .settings(assmSettings)

lazy val backendapi = project.from("backendapi")
  .setName("backendapi")
  .setDescription("backend actors")
  .setInitialCommand("_")
  .configureModule
  .dependsOnProjects(common)
  .enablePlugins(sbtdocker.DockerPlugin, DockerComposePlugin)
  .settings(assmSettings)
  .settings(
    dockerImageCreationTask := docker.value,
    dockerfile in docker := {
      new Dockerfile {
        val mainClassString = (mainClass in Compile).value.get
        val classpath = (fullClasspath in Compile).value
        from("java")
        add(classpath.files, dockerAppPath)
        entryPoint("java", "-cp", s"$dockerAppPath:$dockerAppPath/*", s"$mainClassString")
      }
    },
    imageNames in docker := Seq(ImageName(
      repository = name.value.toLowerCase,
      tag = Some("latest"))
    )
  )


lazy val frontendapi = project.from("frontendapi")
  .setName("frontendapi")
  .setDescription("front end")
  .setInitialCommand("_")
  .configureModule
  .dependsOnProjects(common)
  .enablePlugins(sbtdocker.DockerPlugin, DockerComposePlugin)
  .settings(assmSettings)
  .settings(
    dockerImageCreationTask := docker.value,
    dockerfile in docker := {
      new Dockerfile {
        val mainClassString = (mainClass in Compile).value.get
        val classpath = (fullClasspath in Compile).value
        from("java")
        add(classpath.files, dockerAppPath)
        entryPoint("java", "-cp", s"$dockerAppPath:$dockerAppPath/*", s"$mainClassString")
      }
    },
    imageNames in docker := Seq(ImageName(
      repository = name.value.toLowerCase,
      tag = Some("latest"))
    )
  )

//lazy val first = project.from("first")
//  .setName("first")
//  .setDescription("First project")
//  .setInitialCommand("first._")
//  .configureModule
//  .configureIntegrationTests
//  .configureFunctionalTests
//  .configureUnitTests
//  .dependsOnProjects(common)
//  .settings(assmSettings)
//  .settings(mainClass in (Compile, run) := Some("pl.combosolutions.first.First"))
//
//lazy val second = project.from("second")
//  .setName("second")
//  .setDescription("Second project")
//  .setInitialCommand("second._")
//  .configureModule
//  .configureIntegrationTests
//  .configureFunctionalTests
//  .configureUnitTests
//  .dependsOnProjects(common)
//  .settings(assmSettings)
//  .settings(mainClass in (Compile, run) := Some("pl.combosolutions.first.Second"))
