import sbt._
import Dependencies._

object Dependencies {

  // scala version
  val scalaVersion = "2.11.8"

  // resolvers
  val resolvers = Seq(
    Resolver sonatypeRepo "public"
  )

  // functional utils
  val scalaz   : Seq[ModuleID]        = Seq("org.scalaz" %% "scalaz-core" % "7.2.6")
  val scalazConcurrent: Seq[ModuleID] = Seq("org.scalaz" %% "scalaz-concurrent" % "7.2.6")
  val scalazContrib : Seq[ModuleID]   = Seq("org.typelevel" %% "scalaz-contrib-210" % "0.2" excludeAll ExclusionRule("org.scalaz"))

  // command line
  val scopt : Seq[ModuleID]= Seq("com.github.scopt" %% "scopt" % "3.5.0")

  // logging
  val logback : Seq[ModuleID]= Seq("ch.qos.logback" % "logback-classic" % "1.1.7")

  // testing
  val mockito : Seq[ModuleID]   = Seq("org.mockito" % "mockito-core" % "1.10.19")
  val spec2  : Seq[ModuleID]    = Seq("org.specs2" %% "specs2" % "3.7")
  val spec2Core : Seq[ModuleID] = Seq("org.specs2" %% "specs2-core" % "3.8.5")
  val spec2JUnit: Seq[ModuleID] = Seq("org.specs2" %% "specs2-junit" % "3.8.5")

  val akka :Seq[ModuleID]  = Seq(
    "com.typesafe.akka" %% "akka-actor" % "2.4.17",
    "com.typesafe.akka" %% "akka-cluster" % "2.4.17",
    "com.typesafe.akka" %% "akka-cluster-metrics" % "2.4.17",
    "com.typesafe.akka" %% "akka-cluster-sharding" % "2.4.17",
    "com.typesafe.akka" %% "akka-cluster-tools" % "2.4.17",
    "com.typesafe.akka" %% "akka-multi-node-testkit" % "2.4.17",
    "com.typesafe.akka" %% "akka-remote" % "2.4.17",
    "com.typesafe.akka" %% "akka-slf4j" % "2.4.17",
    "com.typesafe.akka" %% "akka-testkit" % "2.4.17",
    "com.typesafe.akka" %% "akka-http" % "10.0.4",
    "com.typesafe.akka" %% "akka-http-spray-json" % "10.0.4",
    "commons-codec" % "commons-codec" % "1.10",
    "com.datastax.cassandra"  % "cassandra-driver-core" % "3.1.4"  exclude("org.xerial.snappy", "snappy-java"),
    "org.xerial.snappy"       % "snappy-java"           % "1.1.2.6",
    "com.amazonaws" % "aws-java-sdk-s3" % "1.11.104"
  )

}

trait Dependencies {

  val scalaVersionUsed = scalaVersion

  // resolvers
  val commonResolvers = resolvers

  val mainDeps : Seq[ModuleID] = scalaz ++ scalazConcurrent ++ scalazContrib ++ scopt ++ logback ++ akka

  val testDeps :Seq[ModuleID] = mockito ++ spec2 ++ spec2Core ++ spec2JUnit

  implicit class ProjectRoot(project: Project) {

    def root: Project = project in file(".")
  }

  implicit class ProjectFrom(project: Project) {

    private val commonDir = "modules"

    def from(dir: String): Project = project in file(s"$commonDir/$dir")
  }

  implicit class DependsOnProject(project: Project) {

    val dependsOnCompileAndTest = "test->test;compile->compile"

    def dependsOnProjects(projects: Project*): Project =
      project dependsOn (projects.map(_ % dependsOnCompileAndTest): _*)
  }
}
